@@ -42,10 +42,18 @@ submit_properties = {
         "type": ["number", "null"],
         "description": "number of nodes for the job.",
     },
+    "option_flags": {
+        "type": "object",
+        "description": "option flags (dict) or key value pairs for flux.",
+    },
     "exclusive": {
         "type": ["boolean", "null"],
         "description": "ask for exclusive nodes for the job.",
     },
+    "is_launcher": {
+        "type": ["boolean", "null"],
+        "description": "indicate the command is for a launcher (e.g., nextflow, snakemake)",
+    },
 }
 
 job_submit_schema = {
